;Example POOL program, you may use it,
;modify and distribute under GPL license.
;Authors: R&D Independent Team
;Source: www.pool-rnd.com

ht pu fence setr 0.1
(setpc "purple 50)
shared "fb
"fb := newarray 10
repeat count :fb [
  :fb,repcount := (array rnorm 200 rnorm 100 (runif 20 120))
  setxy :fb,repcount,1 :fb,repcount,2
  arc 360 :fb,repcount,3
]
(setpc "white 2)

;Play with the formula to change the field:
; :dx and :dy are x and y distances to the source,
; :r is the range of the force.
to force :r :dx :dy
  let "d sqrt :dx^2 + :dy^2
  let "w (:d / :r)^2
  let "g 1.5 * (0.5 - (1.5 / exp :w^3)) / (-0.5 * (:w + 1) * :d)
  op array :dx * :g :dy * :g
end

to sum_forces :x :y
  let "f {1 0}
  foreach "p :fb [
    let "g force :p,3 (:p,1 - :x) (:p,2 - :y)
    :f,1 += :g,1
    :f,2 += :g,2
  ]
  output :f
end

to particle :mass [:loss 0.02]

  let "speed (runif 5 15)
  to onsignalstep
    let "dt 0.5
    let "f sum_forces xcor ycor
    let "mdir movedir
    :mdir,1 := :mdir,1 * :speed + :f,1 * :dt / :mass
    :mdir,2 := :mdir,2 * :speed + :f,2 * :dt / :mass
    "speed := sqrt (:mdir,1)^2 + (:mdir,2)^2
    if :speed > 50 ["speed := 50]
    setmovedir :mdir alignh
    let "ds :speed * :dt
    "speed -= :loss * :ds
    if :speed < 0 ["speed := 0]
    fd :ds
  end

  to onwallcollided :vdir :drest :wall
    let "x :pool_inf,"wall_left + 1
    let "y (runif :pool_inf,"wall_bottom + 2 :pool_inf,"wall_top - 2)
    pu setxy :x :y pd
    setmovedir {1 0} alignh
  end

  ht
  setxy :pool_inf,"wall_left + 1 (random -150 150)
  setmovedir {1 0}
  pd
end

to background :x :y
  let "c newarray 3
  let "f sum_forces :x :y
  op (array 30 * sqrt :f,1^2 + :f,2^2 ~
  20 * :f,2 20 * :f,1)
end
print "|wait a moment, please...|
(render $background)
print "done

"ex := (newturtles 200 $particle 2)
"t := timer "step 7


